let scene, camera, renderer, qrTexture, shaderMaterial;
let isWebGLAvailable = false;

function checkWebGLAvailability() {
    try {
        const canvas = document.createElement('canvas');
        isWebGLAvailable = !!(window.WebGLRenderingContext && (canvas.getContext('webgl') || canvas.getContext('experimental-webgl')));
    } catch(e) {
        isWebGLAvailable = false;
    }
}

function init() {
    checkWebGLAvailability();

    if (isWebGLAvailable) {
        initWebGL();
    } else {
        initFallback();
    }
}

function initWebGL() {
    scene = new THREE.Scene();
    camera = new THREE.OrthographicCamera(-1, 1, 1, -1, 0.1, 10);
    camera.position.z = 1;

    const canvas = document.getElementById('qr-canvas');
    renderer = new THREE.WebGLRenderer({ canvas: canvas, antialias: true });
    renderer.setSize(800, 800);

    const geometry = new THREE.PlaneGeometry(2, 2);

    const vertexShader = `
        varying vec2 vUv;
        void main() {
            vUv = uv;
            gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);
        }
    `;

    const fragmentShader = `
        uniform sampler2D qrTexture;
        uniform float iTime;
        uniform vec2 iResolution;
        varying vec2 vUv;

        vec3 palette(float t) {
            vec3 a = vec3(0.5, 0.5, 0.5);
            vec3 b = vec3(0.5, 0.5, 0.5);
            vec3 c = vec3(1.0, 1.0, 1.0);
            vec3 d = vec3(0.263, 0.416, 0.557);
            return a + b*cos(6.28318*(c*t+d));
        }

        void main() {
            vec2 uv = (vUv * 2.0 - 1.0) * vec2(iResolution.x/iResolution.y, 1.0);
            vec2 uv0 = uv;
            vec3 finalColor = vec3(0.0);
            
            for (float i = 0.0; i < 4.0; i++) {
                uv = fract(uv * 1.5) - 0.5;
                float d = length(uv) * exp(-length(uv0));
                vec3 col = palette(length(uv0) + i*0.4 + iTime*0.4);
                d = sin(d*8. + iTime)/8.;
                d = abs(d);
                d = pow(0.01 / d, 1.2);
                finalColor += col * d;
            }

            vec4 qrColor = texture2D(qrTexture, vUv);
            float qrMask = 1.0 - qrColor.r;  // Assuming black QR on white background
            finalColor = mix(finalColor, vec3(0.0), qrMask);
            
            gl_FragColor = vec4(finalColor, 1.0);
        }
    `;

    shaderMaterial = new THREE.ShaderMaterial({
        uniforms: {
            qrTexture: { value: null },
            iTime: { value: 0.0 },
            iResolution: { value: new THREE.Vector2(800, 800) }
        },
        vertexShader: vertexShader,
        fragmentShader: fragmentShader
    });

    const plane = new THREE.Mesh(geometry, shaderMaterial);
    scene.add(plane);

    animate();
}

function initFallback() {
    console.log("WebGL not available, using fallback rendering");
    const canvas = document.getElementById('qr-canvas');
    canvas.width = 800;
    canvas.height = 800;
}

function animate() {
    if (isWebGLAvailable) {
        requestAnimationFrame(animate);
        shaderMaterial.uniforms.iTime.value += 0.01;
        renderer.render(scene, camera);
    }
}

function generateQRCode(url) {
    const qr = qrcode(0, 'M');
    qr.addData(url);
    qr.make();
    const size = qr.getModuleCount();
    const canvas = document.createElement('canvas');
    canvas.width = canvas.height = size * 4;  // Increase resolution
    const ctx = canvas.getContext('2d');
    ctx.scale(4, 4);  // Scale up for higher resolution

    for (let row = 0; row < size; row++) {
        for (let col = 0; col < size; col++) {
            ctx.fillStyle = qr.isDark(row, col) ? '#000' : '#fff';
            ctx.fillRect(col, row, 1, 1);
        }
    }

    if (isWebGLAvailable) {
        qrTexture = new THREE.CanvasTexture(canvas);
        qrTexture.minFilter = THREE.LinearFilter;
        qrTexture.magFilter = THREE.LinearFilter;
        shaderMaterial.uniforms.qrTexture.value = qrTexture;
    } else {
        const destCanvas = document.getElementById('qr-canvas');
        const destCtx = destCanvas.getContext('2d');
        destCtx.clearRect(0, 0, destCanvas.width, destCanvas.height);
        destCtx.drawImage(canvas, 0, 0, destCanvas.width, destCanvas.height);
    }
}

init();

document.getElementById('generate-btn').addEventListener('click', () => {
    const url = document.getElementById('url-input').value;
    if (url) {
        generateQRCode(url);
    } else {
        alert('Please enter a URL');
    }
});
